==> common_bloc-4.2.1.tar.gz: lib/src/cubit/rest/rest_cubit.dart <==
import 'package:bloc/bloc.dart';
import 'package:common_bloc/common_bloc.dart';
import 'package:common_bloc/src/common/source/data_source.dart';

///RestCubit - A cubit to make request to a REST API
///
///GET, POST, PUT, PATCH, DELETE available
class RestCubit extends Cubit<RestState> {
  ///Main constructor
  RestCubit(String baseUrl, {List<Interceptor>? interceptors})
      : super(RestState.uninitialized()) {
    _restDataSource = RestDataSource(
      baseURL: baseUrl,
      client: Dio()
        ..interceptors.addAll([if (interceptors != null) ...interceptors]),
    );
  }

  late RestDataSource _restDataSource;

  ///Get current base url
  String get currentBaseUrl => _restDataSource.currentBaseUrl;

  set currentBaseUrl(String newBaseUrl) =>
      _restDataSource.currentBaseUrl = newBaseUrl;

  ///GET request to API
  ///
  ///Perform a GET request to API with specific
  ///[path], you can send the [headers], [params]
  ///and a [fromJson] method to parse the result
  ///json to a specific object.
  Future<void> get(
    String path, {
    Function(Map<String, dynamic>)? fromJson,
    Map<String, String>? headers,
    Map<String, String>? params,
    bool withLoading = true,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.get(
        path,
        fromJson: fromJson,
        params: params,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///POST request to API
  ///
  ///Perform a POST request to API with specific
  ///[path] and a [body]. You can send the [headers], [contentType]
  ///and a [fromJson] method to parse the result json to
  ///a specific object.
  Future<void> post(
    String path, {
    Function(Map<String, dynamic>)? fromJson,
    Map<String, String>? headers,
    Map<String, dynamic>? body,
    String? contentType,
    bool withLoading = true,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.post(
        path,
        body: body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///PUT request to API
  ///
  ///Perform a PUT request to API with specific
  ///[path] and a [body]. You can send the [headers], [contentType]
  ///and a [fromJson] method to parse the result json to
  ///a specific object.
  Future<void> put(
    String path, {
    Function(Map<String, dynamic>)? fromJson,
    Map<String, String>? headers,
    Map<String, dynamic>? body,
    String? contentType,
    bool withLoading = true,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.put(
        path,
        body: body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///PATCH request to API
  ///
  ///Perform a PATCH request to API with specific
  ///[path] and a [body]. You can send the [headers], [contentType]
  ///and a [fromJson] method to parse the result json to
  ///a specific object.
  Future<void> patch(
    String path, {
    Function(Map<String, dynamic>)? fromJson,
    Map<String, String>? headers,
    Map<String, dynamic>? body,
    String? contentType,
    bool withLoading = true,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.patch(
        path,
        body: body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///DELETE request to API
  ///
  ///Perform a DELETE request to API with specific
  ///[path]. You can send the [headers] for request.
  Future<void> delete(
    String path, {
    Function(Map<String, dynamic>)? fromJson,
    Map<String, String>? headers,
    bool withLoading = true,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.delete(path, headers: headers);
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///POST request with FormData content.
  ///
  ///Perform a POST request to API with specific
  ///[path] and a [formData] as data.
  ///
  ///Usefull for uploading files
  ///
  ///You can specify the [headers] for request.
  Future<void> formData(
    String path, {
    required FormData body,
    Function(Map<String, dynamic>)? fromJson,
    Function(int, int)? onProgressChanged,
    Map<String, String>? headers,
    String? contentType,
    bool withLoading = true,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.formData(
        path,
        body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
        onProgressChanged: onProgressChanged,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///Clear RestState
  ///
  ///Restart the state to uninitialized
  ///
  ///Usefull for reusing the cubit without construct
  ///it again.
  void clear() => emit(RestState.uninitialized());
}

==> common_bloc-4.2.3.tar.gz: lib/src/cubit/rest/rest_cubit.dart <==
import 'package:bloc/bloc.dart';
import 'package:common_bloc/common_bloc.dart';
import 'package:common_bloc/src/common/source/data_source.dart';

///RestCubit - A cubit to make request to a REST API
///
///GET, POST, PUT, PATCH, DELETE available
class RestCubit extends Cubit<RestState> {
  ///Main constructor
  RestCubit(String baseUrl, {List<Interceptor>? interceptors})
      : super(RestState.uninitialized()) {
    _restDataSource = RestDataSource(
      baseURL: baseUrl,
      client: Dio()
        ..interceptors.addAll([if (interceptors != null) ...interceptors]),
    );
  }

  late RestDataSource _restDataSource;

  ///Get current base url
  String get currentBaseUrl => _restDataSource.currentBaseUrl;

  set currentBaseUrl(String newBaseUrl) =>
      _restDataSource.currentBaseUrl = newBaseUrl;

  ///GET request to API
  ///
  ///Perform a GET request to API with specific
  ///[path], you can send the [headers], [params]
  ///and a [fromJson] method to parse the result
  ///json to a specific object.
  Future<void> get(
    String path, {
    Map<String, String>? headers,
    Map<String, String>? params,
    bool withLoading = true,
    dynamic Function(Map<String, dynamic>)? fromJson,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.get(
        path,
        fromJson: fromJson,
        params: params,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///POST request to API
  ///
  ///Perform a POST request to API with specific
  ///[path] and a [body]. You can send the [headers], [contentType]
  ///and a [fromJson] method to parse the result json to
  ///a specific object.
  Future<void> post(
    String path, {
    Map<String, String>? headers,
    Map<String, dynamic>? body,
    String? contentType,
    bool withLoading = true,
    dynamic Function(Map<String, dynamic>)? fromJson,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.post(
        path,
        body: body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///PUT request to API
  ///
  ///Perform a PUT request to API with specific
  ///[path] and a [body]. You can send the [headers], [contentType]
  ///and a [fromJson] method to parse the result json to
  ///a specific object.
  Future<void> put(
    String path, {
    Map<String, String>? headers,
    Map<String, dynamic>? body,
    String? contentType,
    bool withLoading = true,
    dynamic Function(Map<String, dynamic>)? fromJson,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.put(
        path,
        body: body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///PATCH request to API
  ///
  ///Perform a PATCH request to API with specific
  ///[path] and a [body]. You can send the [headers], [contentType]
  ///and a [fromJson] method to parse the result json to
  ///a specific object.
  Future<void> patch(
    String path, {
    Map<String, String>? headers,
    Map<String, dynamic>? body,
    String? contentType,
    bool withLoading = true,
    dynamic Function(Map<String, dynamic>)? fromJson,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.patch(
        path,
        body: body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///DELETE request to API
  ///
  ///Perform a DELETE request to API with specific
  ///[path]. You can send the [headers] for request.
  Future<void> delete(
    String path, {
    Map<String, String>? headers,
    bool withLoading = true,
    dynamic Function(Map<String, dynamic>)? fromJson,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.delete(path, headers: headers);
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///POST request with FormData content.
  ///
  ///Perform a POST request to API with specific
  ///[path] and a [formData] as data.
  ///
  ///Usefull for uploading files
  ///
  ///You can specify the [headers] for request.
  Future<void> formData(
    String path, {
    Map<String, String>? headers,
    String? contentType,
    bool withLoading = true,
    dynamic Function(Map<String, dynamic>)? fromJson,
    required FormData body,
    void Function(int, int)? onProgressChanged,
  }) async {
    if (withLoading) emit(RestState.loading());
    try {
      final result = await _restDataSource.formData(
        path,
        body,
        contentType: contentType,
        fromJson: fromJson,
        headers: headers,
        onProgressChanged: onProgressChanged,
      );
      emit(
        RestState.loaded(
          data: result['data'],
          headers: result['headers'] as Map<String, List<String>>,
          lastPath: path,
          timestamp: DateTime.now().toIso8601String(),
        ),
      );
    } on ResponseException catch (e) {
      emit(RestState.error(humanMessage: e.humanMessage, message: e.message));
    }
  }

  ///Clear RestState
  ///
  ///Restart the state to uninitialized
  ///
  ///Usefull for reusing the cubit without construct
  ///it again.
  void clear() => emit(RestState.uninitialized());
}
